# -*- coding: utf-8 -*-
import json
import os
import re
import sys

import responses
from crabpy.gateway.exception import GatewayRuntimeException

try:
    from unittest.mock import Mock
except ImportError:
    from mock import Mock  # pragma: no cover

# gewest en provincies

gewest_mock = Mock()

antwerpen = Mock()
antwerpen.naam = "Antwerpen"
antwerpen.gewest = gewest_mock
antwerpen.niscode = 10000

vlbranbant = Mock()
vlbranbant.naam = "Vlaams-Brabant"
vlbranbant.gewest = gewest_mock
vlbranbant.niscode = 20001

wvlaanderen = Mock()
wvlaanderen.naam = "West-Vlaanderen"
wvlaanderen.gewest = gewest_mock
wvlaanderen.niscode = 30000

ovlaanderen = Mock()
ovlaanderen.naam = "Oost-Vlaanderen"
ovlaanderen.gewest = gewest_mock
ovlaanderen.niscode = 40000

limburg = Mock()
limburg.naam = "Limburg"
limburg.gewest = gewest_mock
limburg.niscode = 70000

# Knokke-Heist

pk_8300_mock = Mock()
pk_8300_mock.id = 8300

pk_8301_mock = Mock()
pk_8301_mock.id = 8301

ns_num_6_sub_mock = Mock()
ns_num_6_sub_mock.id = 1441952
ns_num_6_sub_mock.huisnummer_id = 270059
ns_num_6_sub_mock.subadres = "1"

ns_num_69_sub_mock = Mock()
ns_num_69_sub_mock.id = 1442188
ns_num_69_sub_mock.huisnummer_id = 882821
ns_num_69_sub_mock.subadres = "11"

ns_num_6_mock = Mock()
ns_num_6_mock.id = 270059
ns_num_6_mock.straat_id = 48086
ns_num_6_mock.huisnummer = "6"
ns_num_6_mock.subadressen = [ns_num_6_sub_mock]
ns_num_6_mock.postkanton = pk_8300_mock

ns_num_68_mock = Mock()
ns_num_68_mock.id = 887821
ns_num_68_mock.straat_id = 48086
ns_num_68_mock.huisnummer = "68"
ns_num_68_mock.subadressen = [ns_num_69_sub_mock]
ns_num_68_mock.postkanton = pk_8300_mock

ns_num_69_mock = Mock()
ns_num_69_mock.id = 882821
ns_num_69_mock.straat_id = 43086
ns_num_69_mock.huisnummer = "69"
ns_num_69_mock.subadressen = []
ns_num_69_mock.postkanton = pk_8300_mock

nieuwstraat_mock = Mock()
nieuwstraat_mock.id = 48086
nieuwstraat_mock.gemeente_id = 191
nieuwstraat_mock.label = "Nieuwstraat"
nieuwstraat_mock.huisnummers = [ns_num_6_mock, ns_num_68_mock]

knokke_mock = Mock()
knokke_mock.id = 191
knokke_mock.naam = "Knokke-Heist"
knokke_mock.niscode = 31043
knokke_mock.provincie = wvlaanderen
knokke_mock.straten = [nieuwstraat_mock]
knokke_mock.postkantons = [pk_8300_mock, pk_8301_mock]

# Lier

pk_2500_mock = Mock()
pk_2500_mock.id = 2500

lier_mock = Mock()
lier_mock.id = 36
lier_mock.naam = "Lier"
lier_mock.niscode = 12021
lier_mock.provincie = antwerpen
lier_mock.straten = []
lier_mock.postkantons = [pk_2500_mock]

# Leuven

leuven_mock = Mock()
leuven_mock.id = 143
leuven_mock.naam = "Leuven"
leuven_mock.niscode = 24062
leuven_mock.provincie = vlbranbant
leuven_mock.straten = []
leuven_mock.postkantons = []

gewest_mock.gemeentes = [knokke_mock, lier_mock, leuven_mock]

# ---------------

gewest_mock_dict = {1: gewest_mock, 2: gewest_mock, 3: gewest_mock}
gemeente_mock_dict = {191: knokke_mock, 36: lier_mock, 143: leuven_mock}
gemeente_niscode_mock_dict = {31043: knokke_mock, 12021: lier_mock, 24062: leuven_mock}
straten_mock_dict = {48086: nieuwstraat_mock}
num_mock_dict = {270059: ns_num_6_mock, 882821: ns_num_69_mock, 887821: ns_num_68_mock}
subadres_mock_dict = {1441952: ns_num_6_sub_mock, 1442188: ns_num_69_sub_mock}


def list_gemeenten(gewest_id):
    if gewest_id in gewest_mock_dict:
        return gewest_mock_dict[gewest_id].gemeentes
    else:
        return None


def get_gemeente_by_id(id):
    if id in gemeente_mock_dict:
        return gemeente_mock_dict[id]
    else:
        raise GatewayRuntimeException("ongeldige gemeente", Mock())


def get_gemeente_by_niscode(niscode):
    if niscode in gemeente_niscode_mock_dict:
        return gemeente_niscode_mock_dict[niscode]
    else:
        raise GatewayRuntimeException("ongeldige gemeente", Mock())


def get_straat_by_id(id):
    if id in straten_mock_dict:
        return straten_mock_dict[id]
    else:
        raise GatewayRuntimeException("ongeldige straat", Mock())


def get_huisnummer_by_id(id):
    if id in num_mock_dict:
        return num_mock_dict[id]
    else:
        raise GatewayRuntimeException("ongeldig huisnummer", Mock())


def get_subadres_by_id(id):
    if id in subadres_mock_dict:
        return subadres_mock_dict[id]
    else:
        raise GatewayRuntimeException("ongeldig subadres", Mock())


def get_postkanton_by_huisnummer(hn_id):
    if hn_id in num_mock_dict:
        return num_mock_dict[hn_id].postkanton
    else:
        return None


def list_postkantons_by_gemeente(gem_id):
    if gem_id in gemeente_mock_dict:
        return gemeente_mock_dict[gem_id].postkantons
    else:
        return None


def list_provincies(gew_id):
    if gew_id == 2:
        return [antwerpen, vlbranbant, wvlaanderen, ovlaanderen, limburg]
    else:
        return []


crab_gateway_mock = Mock()
crab_gateway_mock.get_gemeente_by_id = get_gemeente_by_id
crab_gateway_mock.get_straat_by_id = get_straat_by_id
crab_gateway_mock.get_huisnummer_by_id = get_huisnummer_by_id
crab_gateway_mock.get_subadres_by_id = get_subadres_by_id
crab_gateway_mock.get_postkanton_by_huisnummer = get_postkanton_by_huisnummer
crab_gateway_mock.list_postkantons_by_gemeente = list_postkantons_by_gemeente
crab_gateway_mock.list_gemeenten = list_gemeenten
crab_gateway_mock.list_provincies = list_provincies
crab_gateway_mock.get_gemeente_by_niscode = get_gemeente_by_niscode

afdeling1 = Mock()
afdeling1.id = 66666
afdeling1.naam = "test "
afdeling1.gemeente.id = 12004

afdeling2 = Mock()
afdeling2.id = 24505
afdeling2.naam = "LEUVEN  5 AFD"
afdeling2.gemeente.id = 24062

afdeling3 = Mock()
afdeling3.id = 123456
afdeling3.naam = "test x"
afdeling3.gemeente.id = 31043


def list_kadastrale_afdelingen():
    return [afdeling1, afdeling2, afdeling3]


capakey_gateway_mock = Mock()
capakey_gateway_mock.list_kadastrale_afdelingen = list_kadastrale_afdelingen


def text_(s, encoding='latin-1', errors='strict'):
    """ If ``s`` is an instance of ``binary_type``, return
    ``s.decode(encoding, errors)``, otherwise return ``s``"""
    # True if we are running on Python 3.
    PY3 = sys.version_info[0] == 3
    if PY3:
        binary_type = bytes
    else:
        binary_type = str
    if isinstance(s, binary_type):
        return s.decode(encoding, errors)
    return s


with open(os.path.join(os.path.dirname(__file__), 'fixtures/get_gemeente_results.json'), 'rb') as f:
    get_gemeente_results = json.loads(text_(f.read()))
with open(os.path.join(os.path.dirname(__file__), 'fixtures/get_provincie_results.json'), 'rb') as f:
    get_provincie_results = json.loads(text_(f.read()))


def mock_geozoekdiensten_response(base_url='http://geozoekdienst.en', response_status=200):
    def callback(request):
        resp_body = [{'naam': 'gemeente'}]
        headers = {'content_type': 'application/json'}
        return response_status, headers, json.dumps(resp_body)

    responses.add_callback(
        responses.POST,
        re.compile(r'^({0}).+'.format(base_url)),
        callback=callback)
    return base_url


def mock_geozoekdiensten_get_gemeente_response(len_results, base_url='http://geozoekdienst.en'):
    def callback(request):
        if len_results == 2:
            resp_body = get_gemeente_results
        elif len_results == 1:
            resp_body = [{'naam': 'gemeente', 'id': 'niscode'}]
        else:
            resp_body = []
        headers = {'content_type': 'application/json'}
        return 200, headers, json.dumps(resp_body)

    responses.add_callback(
        responses.POST,
        re.compile(r'^({0}).+'.format(base_url)),
        callback=callback)
    return base_url
